      subroutine mie_ci_bs(freq, temp, lwc, ksca, asca, gsca, pbck)
      

c     Compute the extinction, absorption, asymmetry parameter and
c     backscatter for a given water content of cloud ice in [g/m^3].

c     Input:
c     freq		frequency of radiation [GHz]
c     temp		temperature of particles [K]
c     lwc		water content of cloud ice distribution [g/m**3]

c     Output:
c     ksca		extinction coefficient [1/km]
c     asca		single-scatter albedo []
c     gsca		asymmetry factor []
c     pbck		backscatter phase function/(4*pi) []
c

      implicit none

      integer *4 i
      integer *4 nopt
      
      real *4 freq
      real *4 temp 
      real *4 lwc
      real *4 ksca 
      real *4 asca 
      real *4 gsca
      real *4 pbck

      real *4 wave
      real *4 pi
      real *4 rad
      real *4 densice
      real *4 densliq
      real *4 density
      real *4 lam
      real *4 densi
      real *4 iwctest
      real *4 norm
      real *4 num
      real *4 faa
      real *4 eicere
      real *4 eiceim 
      real *4 qext
      real *4 qsca
      real *4 asym
      real *4 qbsca
      real *4 bext
      real *4 bsca
      real *4 bsym
      real *4 bq11
      
      real *8 xd 
      real *8 freqhzd
      real *8 freqd
      real *8 tempd
      real *8 sald
      real *8 ewatred
      real *8 ewatimd
      real *8 eicered
      real *8 eiceimd
      real *8 qscad
      real *8 qextd
      real *8 asymd
      real *8 qbscad

      complex *8 eice
      complex *8 eair
      complex *8 ei

      complex *16 ewatd
      complex *16 eid
      complex *16 crefd

      data pi/3.14159265/
      data densliq/1.0e+3/
      data densice/0.917e+3/

c      
c     Assign some useful constants
      wave=300./freq
      
c
c     Begin by checking if hydrometeors of this species are present.
c     If not, set scattering parameters to zero and return.
c
      if(lwc .lt. 0.001) then
        ksca=0.
        asca=0.
        gsca=0.
        pbck=0.
        return
      endif

c
c     If hydrometeors are present, initialize the scattering parameters
c
      bext=0.
      bsca=0.
      bsym=0.
      bq11=0.

c
c     Loop over particle sizes:

c     increments of particle radius are 0.005 mm; the particle
c     size distribution is expressed as a particle number density,
c     num, per radius increment.  This distribution is given
c     by the fit to observed cloud ice distributions by
c     Heymsfield and Platt (1984)
c     two options are available:
c     nopt=0:   ice particle mass distributed in spherical
c               volume with diameter equal to maximum crystal
c               dimension (l).
c     nopt=1:   ice particle described as pure ice sphere
c               with same mass as elongated crystal.
      nopt=1

c
c     first compute normalization factor for particle size distribution
      norm=1.
      iwctest=0.
      do i=0,500
        rad=0.0025+0.005*float(i)
        call heymplatt(nopt,temp,lwc,rad,densi,num,norm)
        density=densi
        iwctest=iwctest+
     +   num*(1.e-3)*density*4.*pi*((rad*1.e-1)**3.)/3.*.005
      end do
      norm=iwctest/lwc

      do i=0,500
        rad=0.0025+0.005*float(i)
        xd=dble(2.*pi*rad/wave)

        call heymplatt(nopt,temp,lwc,rad,densi,num,norm)

c
c       complex refractive index of cloud ice
c
        freqd=dble(freq)
        tempd=dble(temp)
        call iceoptic(freqd,tempd,eicered,eiceimd)
        eicere=real(eicered)
        eiceim=real(eiceimd)
        eice=cmplx(eicere,eiceim)
        eair=cmplx(1.0006,0.0)

c       calculate dielectric constant of cloud ice as
c       ice matrix with air inclusions, using
c       Maxwell-Garnett mixing
        faa=1.-((densi*1.e+3)/densice)
        call mg_ellips(faa,eice,eair,ei)
        eid=dcmplx(ei)
        crefd=cdsqrt(eid)

c
c       call Mie program
c 
        call mie_sphere(xd,crefd,qscad,qextd,asymd,qbscad)

        qext=real(qextd)
        qsca=real(qscad)
        asym=real(asymd)
        qbsca=real(qbscad)
        
c
c       integrate over particle size distribution;

        bext=bext+num*qext*pi*rad*rad*(0.005)*1.e-6
        bsca=bsca+num*qsca*pi*rad*rad*(0.005)*1.e-6
        bsym=bsym+num*qsca*asym*pi*rad*rad*(0.005)*1.e-6
        bq11=bq11+num*qbsca*pi*rad*rad*(0.005)*1.e-6

      end do

c
c     check for distribution with very small extinction;
c     set parameters to zero to avoid numerical problems
      if( bext .gt. 1.e-6) then

        ksca=bext
        asca=bsca/bext
        gsca=bsym/bsca
        pbck=bq11/bsca

      else

        ksca=0.
        asca=0.
        gsca=0.
        pbck=0.

      end if

      return
      end
C -----------------------------------------------------------
      subroutine heymplatt(nopt,t,iwc,r,den,numden,norm)

    ! Return particle density and number density
    ! of ice crystals, given temperature, ice water content of
    ! ice crystal distribution, and maximum particle 
    ! dimension.  Follows the empirical relation of 
    ! Heymsfield and Platt (1984).

    ! Input
    ! Two options are available:
    ! nopt=0:   ice particle mass distributed in spherical
    !	 	volume with diameter equal to maximum crystal
    !		dimension (l).
    ! nopt=1:	ice particle described as pure ice sphere
    !		with same mass as elongated crystal.	  
    ! t		temperature [K]
    ! iwc	equivalent water content of cloud ice [g/m**3]
    ! r		particle radius [mm]

    ! Output
    ! den	particle density [g/cm**3]
    ! numden	particle number density [1/m**4]
    ! norm	normalization factor required to contrain
    !		integrated distribution to equal the equivalent
    !		ice water content.  heymplatt must be called
    !		first with norm set equal to 1 to give
    !           proper normalization

    ! Bill Olson  April, 1998

      implicit none

      integer *4 n
      integer *4 nopt

      real *4 t
      real *4 iwc
      real *4 r
      real *4 den
      real *4 numden
      real *4 a(0:4)
      real *4 b(0:4)
      real *4 c(0:4)
      real *4 d(0:4)
      real *4 e(0:4)
      real *4 pi
      real *4 suma
      real *4 sumb
      real *4 sumc
      real *4 sumd
      real *4 sume
      real *4 tc
      real *4 mass
      real *4 l
      real *4 l0
      real *4 b1
      real *4 b2
      real *4 a1
      real *4 a2
      real *4 n100diwc
      real *4 n1000diwc
      real *4 lc
      real *4 dc
      real *4 densice
      real *4 norm
      real *4 dldr

      data a/-1.1430e+1,-7.3892e-1,-1.8647e-2,-1.4045e-4,0.0e+0/
      data b/1.8940e+1,2.7658e0,1.2833e-1,2.7750e-3,2.2994e-5/
      data c/-1.0159e+1,-1.4538e0,-1.3511e-2,1.1318e-3,2.2360e-5/
      data d/1.6764e+1,-1.5072e-1,-1.9713e-2,-3.5051e-4,-1.6727e-6/
      data e/1.5508e+2,1.8377e+1,8.5312e-1,1.6879e-2,1.1873e-4/
      data pi/3.14159265/
      data densice/0.917/


    ! Temperature [C]
      tc=t-273.16

    ! Note: empirical formulae only apply to range
    ! -60 < tc < -20
      if(tc .gt. -20.) tc=-20.
      if(tc .lt. -60.) tc=-60.

    ! Calculate maximum particle dimension [microns]
        if(nopt .eq. 0) then
          l=2.*r*1.e+3
        else
          mass=densice*4.*pi*((r*1.e-1)**3.)/3.
          if(2.*r .le. 0.3) then
            l=((mass/(densice*(3.*sqrt(3.)/2.)*
     +       ((0.5/2.)**2.)*1.e-3))**.33333)*
     +       1.e+3
            dldr=(4./3.)*(1.e-3)*densice*pi*(mass**(-.66666))*r*r/
     +       ((densice*(3.*sqrt(3.)/2.)*((0.5/2.)**2.)*1.e-3)**.33333)
          else
            l=((mass/(densice*(3.*sqrt(3.)/2.)*((.2/2.)**2.)*1.e-3))
     +       **.55249)*1.e+3
            dldr=(4./1.82)*(1.e-3)*densice*pi*(mass**(1./1.82-1.))*r*r/
     +      ((densice*(3.*sqrt(3.)/2.)*((0.2/2.)**2.)*1.e-3)**(1./1.82))
          end if
        end if

        suma=0.
        sumb=0.
        sumc=0.
        sumd=0.
        sume=0.
        do n=0,4
          suma=suma+a(n)*(tc**n)
          sumb=sumb+b(n)*(tc**n)
          sumc=sumc+c(n)*(tc**n)
          sumd=sumd+d(n)*(tc**n)
          sume=sume+e(n)*(tc**n)
        end do
        b1=suma
        b2=sumb

    !  Liou's fit of b2 fails at low temperature; 
    !  since Heymsfield and Platt (1984) data indicate
    !  a nearly constant value of -4., we use it here.
         b2=-4.

        if(tc .ge. -37.5) then
          n100dIWC=exp(sumc)
        else
          n100dIWC=exp(sumd)
        end if
        n1000dIWC=sume

        a1=n100diwc/(100.**b1)
        a2=n1000diwc/(1000.**b2)
        l0=(a2/a1)**(1./(b1-b2))

        if(nopt .eq. 0) then
          if(l .le. l0) then
            numden=(1.e+6)*2.*a1*(l**b1)*iwc/norm
          else
            numden=(1.e+6)*2.*a2*(l**b2)*iwc/norm
          end if
        else
          if(l .le. l0) then
            numden=(1.e+6)*dldr*a1*(l**b1)*iwc/norm
          else
            numden=(1.e+6)*dldr*a2*(l**b2)*iwc/norm
          end if
        end if

    ! Compute particle density
    ! Assume randomly oriented hexagonal column with
    ! width/length relationship determined
    ! from Heymsfield's empirical relation
        if(nopt .eq. 0) then
          if(l*1.e-3 .le. 0.3) then
            dc=0.5*(l*1.e-3)*1.e-1
            lc=l*1.e-4
          else
            dc=0.2*((l*1.e-3)**0.41)*1.e-1
            lc=l*1.e-4
          end if
          mass=densice*(3.*sqrt(3.)/2.)*((dc/2.)**2.)*lc
          den=mass/(4.*pi*((r*1.e-1)**3.)/3.)
        else
          den=0.917
        end if

      return
      end

